@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2012, 2013, 2014 Ivan Cukic <ivan.cukic(at)kde.org>
+ *   Copyright (C) 2012, 2013, 2014, 2015 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -32,7 +32,6 @@
 #include <QModelIndex>
 
 // KDE
-#include <klocalizedstring.h>
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <kdirwatch.h>
@@ -45,14 +44,15 @@
 
 // Local
 #include "utils/remove_if.h"
+#define ENABLE_QJSVALUE_CONTINUATION
 #include "utils/continue_with.h"
-#include "utils_p.h"
+#include "utils/model_updaters.h"
 
 using kamd::utils::continue_with;
 
 
 namespace KActivities {
-namespace Models {
+namespace Imports {
 
 class ActivityModel::Private {
 public:
@@ -191,23 +191,22 @@ public:
 
         QString backgroundFromConfig(const KConfigGroup &config) const
         {
-            // Trying for the wallpaper
-            auto wallpaper = config
-                .group("Wallpaper")
-                .group("General")
-                .readEntry("Image", QString());
-
-            if (!wallpaper.isEmpty()) {
-                return wallpaper;
-
-            } else {
-                auto backgroundColor = config
-                    .group("Wallpaper")
-                    .group("General")
-                    .readEntry("Color", QColor(0, 0, 0));
+            auto wallpaperPlugin = config.readEntry("wallpaperplugin");
+            auto wallpaperConfig = config.group("Wallpaper").group(wallpaperPlugin).group("General");
 
+            if (wallpaperConfig.hasKey("Image")) {
+                // Trying for the wallpaper
+                auto wallpaper = wallpaperConfig.readEntry("Image", QString());
+                if (!wallpaper.isEmpty()) {
+                    return wallpaper;
+                }
+            }
+            if (wallpaperConfig.hasKey("Color")) {
+                auto backgroundColor = wallpaperConfig.readEntry("Color", QColor(0, 0, 0));
                 return backgroundColor.name();
             }
+
+            return QString();
         }
 
         void reload(bool fullReload)
@@ -302,14 +301,15 @@ ActivityModel::~ActivityModel()
 QHash<int, QByteArray> ActivityModel::roleNames() const
 {
     return {
-        {Qt::DisplayRole,    "name"},
-        {Qt::DecorationRole, "icon"},
+        {Qt::DisplayRole,     "name"},
+        {Qt::DecorationRole,  "icon"},
 
-        {ActivityState,      "state"},
-        {ActivityId,         "id"},
-        {ActivityIcon,       "iconSource"},
-        {ActivityBackground, "background"},
-        {ActivityCurrent,    "current"}
+        {ActivityState,       "state"},
+        {ActivityId,          "id"},
+        {ActivityIcon,        "iconSource"},
+        {ActivityDescription, "description"},
+        {ActivityBackground,  "background"},
+        {ActivityCurrent,     "current"}
     };
 }
 
@@ -355,7 +355,7 @@ void ActivityModel::onActivityRemoved(const QString &id)
 
 void ActivityModel::onCurrentActivityChanged(const QString &id)
 {
-    Q_UNUSED(id)
+    Q_UNUSED(id);
 
     for (const auto &activity: m_shownActivities) {
         Private::emitActivityUpdated(this, m_shownActivities, activity->id(),
@@ -376,12 +376,16 @@ ActivityModel::InfoPtr ActivityModel::registerActivity(const QString &id)
     } else {
         auto activityInfo = std::make_shared<Info>(id);
 
-        connect(activityInfo.get(), &Info::nameChanged,
-                this,               &ActivityModel::onActivityNameChanged);
-        connect(activityInfo.get(), &Info::iconChanged,
-                this,               &ActivityModel::onActivityIconChanged);
-        connect(activityInfo.get(), &Info::stateChanged,
-                this,               &ActivityModel::onActivityStateChanged);
+        auto ptr = activityInfo.get();
+
+        connect(ptr,  &Info::nameChanged,
+                this, &ActivityModel::onActivityNameChanged);
+        connect(ptr,  &Info::descriptionChanged,
+                this, &ActivityModel::onActivityDescriptionChanged);
+        connect(ptr,  &Info::iconChanged,
+                this, &ActivityModel::onActivityIconChanged);
+        connect(ptr,  &Info::stateChanged,
+                this, &ActivityModel::onActivityStateChanged);
 
         m_knownActivities.insert(InfoPtr(activityInfo));
 
@@ -456,27 +460,20 @@ void ActivityModel::hideActivity(const QString &id)
     }
 }
 
-void ActivityModel::onActivityNameChanged(const QString &name)
-{
-    Q_UNUSED(name)
+#define CREATE_SIGNAL_EMITTER(What, Role)                                      \
+    void ActivityModel::onActivity##What##Changed(const QString &)             \
+    {                                                                          \
+        Private::emitActivityUpdated(this, m_shownActivities, sender(), Role); \
+    }
 
-    Private::emitActivityUpdated(this, m_shownActivities, sender(),
-                                 Qt::DisplayRole);
-}
+CREATE_SIGNAL_EMITTER(Name, Qt::DisplayRole)
+CREATE_SIGNAL_EMITTER(Description, ActivityDescription)
+CREATE_SIGNAL_EMITTER(Icon, Qt::DecorationRole)
 
-void ActivityModel::onActivityIconChanged(const QString &icon)
-{
-    Q_UNUSED(icon)
-
-    Private::emitActivityUpdated(this, m_shownActivities, sender(),
-                                 Qt::DecorationRole);
-}
+#undef CREATE_SIGNAL_EMITTER
 
 void ActivityModel::onActivityStateChanged(Info::State state)
 {
-    // TODO: Implement this properly
-    // auto activity = static_cast<Info*> (sender());
-
     if (m_shownStates.empty()) {
         Private::emitActivityUpdated(this, m_shownActivities, sender(),
                                      ActivityState);
@@ -488,10 +485,6 @@ void ActivityModel::onActivityStateChanged(Info::State state)
             return;
         }
 
-        // qDebug() << m_shownStatesString << "Activity state has changed: "
-        //          << info->id() << " " << info->name()
-        //          << " to: " << state;
-
         if (boost::binary_search(m_shownStates, state)) {
             showActivity(info, true);
         } else {
@@ -572,6 +565,9 @@ QVariant ActivityModel::data(const QModelIndex &index, int role) const
             return icon.isEmpty() ? "preferences-activities" : icon;
         }
 
+    case ActivityDescription:
+        return item->description();
+
     case ActivityCurrent:
         return m_service.currentActivity() == item->id();
 
@@ -586,11 +582,9 @@ QVariant ActivityModel::data(const QModelIndex &index, int role) const
 QVariant ActivityModel::headerData(int section, Qt::Orientation orientation,
                                    int role) const
 {
-    Q_UNUSED(orientation)
-
-    if (section == 0 && role == Qt::DisplayRole) {
-        return i18nc("Header title for activity data model", "Activity");
-    }
+    Q_UNUSED(section);
+    Q_UNUSED(orientation);
+    Q_UNUSED(role);
 
     return QVariant();
 }
@@ -608,19 +602,19 @@ ActivityModel::InfoPtr ActivityModel::findActivity(QObject *ptr) const
     }
 }
 
-// QFuture<void> Controller::setActivityName(id, name)
-void ActivityModel::setActivityName(const QString &id, const QString &name,
-                                    const QJSValue &callback)
-{
-    continue_with(m_service.setActivityName(id, name), callback);
-}
+// QFuture<void> Controller::setActivityWhat(id, value)
+#define CREATE_SETTER(What)                                                    \
+    void ActivityModel::setActivity##What(                                     \
+        const QString &id, const QString &value, const QJSValue &callback)     \
+    {                                                                          \
+        continue_with(m_service.setActivity##What(id, value), callback);       \
+    }
 
-// QFuture<void> Controller::setActivityIcon(id, icon)
-void ActivityModel::setActivityIcon(const QString &id, const QString &icon,
-                                    const QJSValue &callback)
-{
-    continue_with(m_service.setActivityIcon(id, icon), callback);
-}
+CREATE_SETTER(Name)
+CREATE_SETTER(Description)
+CREATE_SETTER(Icon)
+
+#undef CREATE_SETTER
 
 // QFuture<bool> Controller::setCurrentActivity(id)
 void ActivityModel::setCurrentActivity(const QString &id,
@@ -653,7 +647,8 @@ void ActivityModel::startActivity(const QString &id, const QJSValue &callback)
     continue_with(m_service.startActivity(id), callback);
 }
 
-} // namespace Models
+} // namespace Imports
 } // namespace KActivities
 
-#include "activitymodel.moc"
+// #include "activitymodel.moc"
+
